
/*************************************************************/
/* Input the needed info for an em analysis.                 */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr23may87 */ 

#include "local.h"
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "son.h"
#include "em.typ"

double atof();
int comp_real();
extern char prog_name[];

Get_an_param( An ) An_info *An;
/*************************************************************/
/* Get parameters which control the execution of the analysis*/
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr24may87 */ 
{ FILE *in;
  char *Pos,Str[256],An_unit='M';
  Rtyp F1,F2,F3,*Fcurr;
  int i,Nf,Fcnt;

  if ( strcmp(An->file,"stdin")==0)  return( Ask_freq(An) );
  if ( (in=fopen(An->file,"r")) == NULL ) 
  { fprintf(stderr,"\n\nCannot open analysis file -- %s --\n",An->file);
    fprintf(stderr,"Instead, please provide frequencies for analysis...\n\n");
    return( Ask_freq(An) );
  }
  Fcnt = 10;
  Fcurr = An->Freq = (Rtyp *)new(Fcnt*sizeof(Rtyp));
  An->Num_freq = 0;
  An->fmax = -1.0;

  while ( fgets(Str,255,in)!=NULL )
  { Pos = Str;
    while ( *Pos != '\000' ) *Pos++ = toupper(*Pos);
    Pos=Skip_blnks(Str);
    if ( strncmp(Pos,"VER",3)==0 ) ;  /*do nothing with the version number for now*/
    else if ( strncmp(Pos,"MEM",3)==0 ) { if (An->Mem_saver==-1) An->Mem_saver=1; }
    else if ( strncmp(Pos,"MHZ",3)==0 ) An_unit='M';
    else if ( strncmp(Pos,"GHZ",3)==0 ) An_unit='G';
    else if ( strncmp(Pos,"JXY",3)==0 ) An->jxy = 1;
    else if ( strncmp(Pos,"CMIN",4)==0 )
    { Pos = Next_field(Pos);
      An->cmin = fabs( atof( Pos ) );  /*pF*/
    }
    else if ( strncmp(Pos,"LMAX",4)==0 )
    { Pos = Next_field(Pos);
      An->lmax = fabs( atof( Pos ) );  /*nH*/
    }
    else if ( strncmp(Pos,"RMAX",4)==0 )
    { Pos = Next_field(Pos);
      An->rmax = fabs( atof( Pos ) );  /*Ohms*/
    }
    else if ( strncmp(Pos,"KMIN",4)==0 )
    { Pos = Next_field(Pos);
      An->mmin = fabs( atof( Pos ) );
    }
    else if ( strncmp(Pos,"RZER",4)==0 )
    { Pos = Next_field(Pos);
      An->rzero = fabs( atof( Pos ) );  /*Ohms to be used when R=0 N.G.*/
    }
    else if ( strncmp(Pos,"FMAX",4)==0 )
    { Pos = Next_field(Pos);
      An->fmax = fabs( atof( Pos ) );
      if (An_unit=='G')
	An->fmax *= 1000.0;
    }
    else if ( strncmp(Pos,"CYC",3)==0 ) 
    { if (An->Cyc==-1) { An->Cyc = atoi( Next_field(Pos) );
                         if (An->Cyc<1) An->Cyc = 1;
                         if (An->Cyc>5) An->Cyc = 5;
                       }
    }
    else if ( strncmp(Pos,"FRE",3)==0 ) 
    { if ( *(Pos=Next_field(Pos)) == '\0' )
      { fprintf(stderr,"%s: Null FRE line in %s.\n",prog_name,An->file);  exit(-1); }
      F1 = fabs( atof( Pos ) );
      F2 = ( *(Pos=Next_field(Pos)) =='\0' ) ? F1 : fabs( atof( Pos ) );
      F3 = ( *(Pos=Next_field(Pos)) =='\0' ) ? F2-F1 : atof( Pos );
      if (An_unit=='G')
      { F1 *= 1000.0;  F2 *= 1000.0;  F3 *= 1000.0; }
      if ( fabs(F3)<1.0e-6 )
	F3=1.0e-6;
      if (F2>F1)
	F3 = fabs(F3);
      if (F2<F1)
	F3 = -fabs(F3);
      if ( ((Nf = 1+(fabs(F2-F1)+1.0e-7)/fabs(F3))+An->Num_freq) > Fcnt )
      { An->Freq = (Rtyp *)rnew( (char *)An->Freq,(Fcnt += Nf+10)*sizeof(Rtyp));
	Fcurr = An->Freq + An->Num_freq;
      }
      for (i=0; i<Nf; i++,Fcurr++,(An->Num_freq)++)  *Fcurr = F1+i*F3;
    }
    else if ( strncmp(Pos,"ANN",3)==0 )
      add_ann(&(An->ann),&(An->num_ann),Pos);
    else if (*Pos=='!' || *Pos=='\0')
      ;  /*Comment or blank line*/
    else
    { fprintf(stderr,"\nUnrecognized line in analysis file:\n-- %s --\n",Pos);
      fclose(in);  return(-1);
    }
  }
  fclose(in);

  /*Check for no frequencies*/
  if (An->Num_freq <= 0 )
  { fprintf(stderr,"%s: No frequencies specified in %s.\n",prog_name,An->file);
    exit(-1);
  }

  /*Following also done at end of Ask_freq*/

  /*sort the frequencies*/
  qsort( (char *)(An->Freq),An->Num_freq,sizeof(Rtyp),comp_real);

  /*If not already set, set fmax, used for subsectioning*/
  if ( An->fmax <= 0.0 )
    An->fmax = *( An->Freq + An->Num_freq - 1 );

  if (An->Cyc<1 || An->Cyc>5)  An->Cyc = 2;
  if (An->Mem_saver != 1) An->Mem_saver = 0;
  if (An->Funit=='X')  An->Funit = An_unit;

  return(0);

} /*Get_an_param*/

init_an(an)  An_info *an;
/*************************************************************/
/* Initialize all data in an.                                */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/*jcr02aug91 */ 
{
  an->file = NULL;
  an->cans = NULL;
  an->uans = NULL;
  an->jxy = 0;
  an->jans = NULL;
  an->Cyc = 2;
  an->Mem_saver = 0;
  an->nsubs_only = 0;
  an->State = 1;
  an->Num_freq = 0;
  an->Freq = NULL;
  an->Funit = 'M';
  an->rp = an->xp = an->lp = an->cp = NULL;
  an->ann = NULL;
  an->num_ann = 0;
  an->cmin = SPICE_CMIN_DEF;
  an->lmax = SPICE_LMAX_DEF;
  an->rmax = SPICE_RMAX_DEF;
  an->mmin = SPICE_MMIN_DEF;
  an->rzero = SPICE_RZER_DEF;
  an->fmax = -1.0;  /*Indicates value not yet set*/
  an->mode_calc_time = 0;
  an->mat_fill_time = 0;
  an->mat_solve_time = 0;

} /*init_an*/

clear_an(an)  An_info *an;
/*************************************************************/
/* Free up all memeory allocated to an;                      */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/*jcr02aug91 */ 
{
  if (an->file) sFree( (char *)an->file);
  if (an->cans) Mat_close(an->cans);
  if (an->uans) Mat_close(an->uans);
  if (an->jans) Mat_close(an->jans);
  if (an->Freq) sFree( (char *)an->Freq);
  if (an->rp) sFree( (char *)an->rp);
  if (an->xp) sFree( (char *)an->xp);
  if (an->lp) sFree( (char *)an->lp);
  if (an->cp) sFree( (char *)an->cp);
  if (an->ann) clear_ann(&(an->ann),&(an->num_ann));

} /*clear_an*/

comp_real(arg1,arg2)  Rtyp *arg1,*arg2;
/*************************************************************/
/* Compare the Rtyps pointed to and return result.           */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/*jcr08aug87 */ 
{ if (*arg1 > *arg2) return(1);
  if (*arg1 < *arg2) return(-1);
  return(0);
} /*comp_real*/

Ask_freq( An )  An_info *An;
/*************************************************************/
/* Ask which frequencies to be analyzed.                     */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr24may87 */ 
{ int cnt,i,N;  Rtyp F1,F2,F3;
  char str[256];

  if (An->Cyc<1 || An->Cyc>5)  An->Cyc = 2;   /*Do default values*/
  if (An->Funit=='X') An->Funit='M';

  printf("Enter start frequency (%cHz) = ",An->Funit);
  fgets(str,255,stdin);
  cnt = 0;  /*counter needed if running in bkgrnd with non-existant ctl.an*/
  while ( (F1 = atof(str)) <= 0.0 )
  { fprintf(stderr,"%s: Start frequency must be positive.\n",prog_name);
    printf("Enter start frequency (%cHz) = ",An->Funit);
    fgets(str,255,stdin);
    if (cnt++ > 100)
    { fprintf(stderr,"\n\n\n******\n%s: No frequencies for analysis.\n******\n\n",prog_name);
      exit(0);
    }
  }

  printf("Enter stop frequency (%cHz) = ",An->Funit);
  fgets(str,255,stdin);
  cnt = 0;
  while ( (F2 = atof(str)) < F1 )
  { fprintf(stderr,"%s: Stop frequency must be greater than start frequency.\n",prog_name);
    printf("Enter stop frequency (%cHz) = ",An->Funit);
    fgets(str,255,stdin);
    if (cnt++ > 100)
    { fprintf(stderr,"\n\n\n******\n%s: No frequencies for analysis.\n******\n\n",prog_name);
      exit(0);
    }
  }

  printf("Enter frequency step size (%cHz) = ",An->Funit);
  fgets(str,255,stdin);
  cnt = 0;
  while ( (F3 = atof(str)) <= 0.0 )
  { fprintf(stderr,"%s: Step size must be positive.\n",prog_name);
    printf("Enter frequency step size (%cHz) = ",An->Funit);
    fgets(str,255,stdin);
    if (cnt++ > 100)
    { fprintf(stderr,"\n\n\n******\n%s: No frequencies for analysis.\n******\n\n",prog_name);
      exit(0);
    }
  }
  printf("\n");

  if (An->Funit=='G') { F1 *= 1000.0;  F2 *= 1000.0;  F3 *= 1000.0; }
  F2 *= 1.0000001;
  N = fabs(F1-F2) / F3;
  if (N<0)
  { fprintf(stderr,"%s: No frequencies for analysis.\n",prog_name);
    exit(-1);
  }
  An->Num_freq = ++N;
  An->Freq = (Rtyp *)new( N * sizeof(Rtyp) );
  for (i=0; i<N; i++)  *(An->Freq+i) = F1 + i*F3;

  /*Set fmax, used for subsectioning*/
  An->fmax = *( An->Freq + An->Num_freq - 1 );

  if (An->jxy<0 || An->jxy>1) An->jxy=0;      /*Do default values*/
  if (An->Cyc<1 || An->Cyc>5)  An->Cyc = 2;   /*Also done at end of Get_an_param*/
  if (An->Mem_saver != 1) An->Mem_saver = 0;

  return(0);
} /*Ask_freq*/
