

/******************************************************************/
/*            	Unlock program for em and xgeom		  	  */
/* Copyright (c) 1990 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/

/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: date.i
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Handles date entry and conversions.
	
	CONTENTS:
		default_demo_date() - Returns the default demo date.
		convert_to_date() -  Returns date in format compatible 
				     with Keyfile
		get_date() - Extract date then convert to Unix Time
		getnumber() - Determines number in string.
	
	
*******************************************************************/


/* definitions	*/

/* external	*/

/* structures and global/semiglobal variable */

static char *month[12] = {
	"Jan ", "Feb ", "Mar ", "Apr ", "May ", "Jun ", 
	"Jul ", "Aug ", "Sep ", "Oct ", "Nov ", "Dec "
};

static char *mmonth[12] = {
	"jan ", "feb ", "mar ", "apr ", "may ", "jun ",
	"jul ", "aug ", "sept ", "oct ", "nov ", "dec "
};


static char *lmonth[12] = {
	"january ", "february ", "march ", "april ", "may ", "june ",
	"july ", "august ", "september ", "october ", "november ", "december "
};

/*******************************************************************
	NAME: convert_to_date()
	
	DESCRIPTION: 
		Returns date in format compatible with the 
		Key file.  Input is time in seconds for 1970.
	
	RETURNS: Character string of date in the form "Jan 7, 1990"

*******************************************************************/

void
convert_to_date(str,date)
char *str;
Date *date;
{
	/* month already includes a space after month */
	sprintf(str,"%s%d, %d",month[date->month], date->day,date->year);
}


/*******************************************************************
	NAME: get_date()
	
	DESCRIPTION: 
		The date is stored in string date. Extract the date then
		convert to number of seconds from 1990.

		Legal format of dates are:
			Jan 7, 1990
			January 7, 90
			january 7 90

		The string is converted to Unix time and stored in
		cnvt_date.  If a conversion can not be done, an error
		message is printed to stdout.

	RETURNS:
		TRUE - if could do conversion. 
		FALSE - if could not do conversion
		 in str_date.

*******************************************************************/

int
get_date(cnvt_date,date)
Date *cnvt_date;
char *date;
{
	char short_month[5];
	int year;
	int m;
	int length;
	int rc;
	int day;
	struct tm *tm;
	time_t  timeofday;

	/* convert string to lower case for use in comparison */
	tolowerstr(date);

	/*  Remove leading spaces */
	rmspace(date);

	/* get the month */
	for (m =0; m < 12 ; ++m) {
		strcpy(short_month,month[m]);	
		short_month[0] = tolower(short_month[0]);

		length = strlen(lmonth[m]);
		if ( strncmp(lmonth[m],date,length) == 0 ) {
			break;
		}

		length = strlen(mmonth[m]);
		if ( strncmp(mmonth[m],date,length) == 0 ) {
			break;
		}

		length = strlen(short_month);
		if ( strncmp(short_month,date,length) == 0 ) {
			break;
		}
	}

	if ( m >= 12 ) {
		fprintf(stderr,"Improper month\n");
		return (FALSE);
	}

	date += length;

	/*  Remove leading spaces */
	rmspace(date);

	/* get date */
	rc = getnumber(&day,&length,date);
	if ( rc == FALSE ) {
		fprintf(stderr,"Improper date\n");
		return (FALSE);
	}

	if ( day <=0 || day >=32 ) {
		fprintf(stderr,"Date must be between 1 and 31\n");
		return (FALSE);
	}

	date += length;

	/* skip over a comma */
	rmspace(date);
	if ( date[0] == ',' ) {
		++date;
		rmspace(date);
	}

	/* get year */
	if ( *date != '\0' ) {
		rc = getnumber(&year,&length,date);
		if ( rc == FALSE ) {
			fprintf(stderr,"Improper year\n");
			return (FALSE);
		}

		if ( year < 1950 || year > 2100 ) {
			if ( year < 90 || year >100 ) {
				fprintf(stderr,"Improper year\n");
				return (FALSE);
			} else {
				year += 1900;
			}
		}
	} else {
		/* Use this years date */
		timeofday = time( (time_t *)0 );
		tm = localtime( &timeofday );
		year = tm->tm_year + 1900;

		if ( tm->tm_mon > m ) {
			/* date is before today's month, use next year date */
			year += 1;
		} else if ( tm->tm_mon == m ) {
			if (  tm->tm_mday > day ) {
				/* date is before today's date */
				year += 1;
			}
		}
	}

	cnvt_date->day = day;
	cnvt_date->month = m;
	cnvt_date->year = year;

	return (TRUE);
}
/*******************************************************************
	NAME: check_date()
	
	DESCRIPTION: 
		Check that the date given is later than today's date.

	RETURNS:
		TRUE - date is later or equal to today's date.
		FALSE - date is earlier than todays's date.

*******************************************************************/
#define	SECONDS_IN_A_DAY	24*60*60

int
check_date(date)
Date *date;
{
	int year,m,day;
	struct tm *tm;
	time_t  timeofday;
	char *tz;
	char *tzp;
	char tz_var[20];
	char c;
	int hours;
	int sign;
	extern char *getenv();
	extern time_t time();

	year = date->year - 1900;
	m    = date->month;
	day  = date->day;

	/* Make sure that the date that the user specified is later than */
	/* todays date */

	timeofday = time( (time_t *)0 );
	tm = localtime( &timeofday );

	/*
	**	Check that the user has not played with the TZ variable
	**	To make us think this is a different day than it is
	**	Like setting TZ to "TZ=EST240" which would set the clock
	**	back by 10 days.
	*/


	tz = getenv("TZ");
	if ( tz != NULL ) {
		/* the format of TZ is "XXXn" or "XXXnYYY" where */
		/* n is a positive or negative number 		*/
		if ( strlen(tz) > 3 )  { 
			tzp = tz+3;
			hours = 0;
			sign = 1;
			if ( *tzp == '-' )  {
				sign = -1;
				++tzp;
			}
			while ( isdigit(*tzp) ) {
				c = toascii( *tzp );
				hours *= 10;
				hours += c - '0';
				++tzp;
			}
			hours *= sign;
		} else {
			hours = 100;
		}
		if ( hours > 48 || hours < -48 ) {
			/* 
			** TZ does not look reasonable change it 
			** to GMT0, get the time again, then change
			** it back.
			*/
			putenv("TZ=GMT0");
			tm = localtime( &timeofday );
			strcpy(tz_var,"TZ=");
			strncat(tz_var,tz,15);
			putenv(tz_var);
		}
	}

	if ( ( year <  tm->tm_year ) ||
	     ( year == tm->tm_year && m <  tm->tm_mon ) ||
	     ( year == tm->tm_year && m == tm->tm_mon && day < tm->tm_mday ) ) {
		return (FALSE);
	}
	return(TRUE);
}


/*******************************************************************
	NAME: getnumber()
	
	DESCRIPTION: 
		Determines the first number encountered in string.
		If non-encountered, then return error, else return
		the number in num and the number of digits in digits.

	RETURNS:
		TRUE - found a number.
		FALSE - could not find a number.

*******************************************************************/
int
getnumber(num,digits,string)
int *num;
int *digits;
char *string;
{
	int rc;
	char c;

	rc = FALSE;
	*num = 0;
	*digits = 0;
	while ( isdigit(*string) ) {
		rc = TRUE;
		c = toascii( *string );
		*num = *num * 10 + ( c - '0');
		*digits += 1;
		++string;
	}
	return rc;
}
