/* Data definitions for the em analysis */   /*jcr16jul87*/

#ifdef EXCLUDE_VEC

#include "emtyp.h"

#else

#include "vec.h"
#include "mat.h"
#include "complex.h"

#endif

#define MIN_REF_DIST 4
#define DEF_GEO_FILE "cir.geo"
#define DEF_AN_FILE "ctl.an"
#define DEF_NDBED_FILE "cir.nd"
#define DEF_DBED_FILE "cir.d"
#define DEF_JXY_FILE "cir.jxy"
#define DEF_SPICE_FILE "spice.lc"
#define DEF_SPICET_FILE "spice.lct"
#define DEF_CIR_NAME ""

/*Convert float coord to integer x or y coord*/
#define intx(A) ( ( 1 + (int)(A) ) & ~1 )
#define inty(A) ( ( 1 + (int)(A) ) & ~1 )
#define intx_odd(A) ( ( (int)(A) ) | 1 )
#define inty_odd(A) ( ( (int)(A) ) | 1 )

/* MKS units used at all times */

/*Flags for converting floating pt outline to integer outline (mode)*/
#define SIMPLE 0
#define STAIRCASE 1
#define DIAGONAL 2

typedef struct { int n;           /*Number of points in integer list*/
                 int size;        /*Number of integers per point*/
                 int max;         /*Number of points allocated*/
                 int *ipt0;       /*Head of points list*/
                 int (*cmp_func)(); /*Pointer to compare function for sorting*/
                 int (*prev_cmp)(); /*If prev_cmp!=cmp_func, sort b4 bsearch*/
                 int (*rules)();  /*Pointer to function for checking point OK*/
               } Ipts;

typedef struct { int n;           /*Number of points in polygon*/
		 int max;         /*Max x,y points memory allocated*/
		 Rtyp *xy0;       /*List of x,y vertices*/
                 int mode;        /*tri_fill, cor_fill or none*/
                 int icond;       /*cond index associated with poly, used in*/
                 int xmin;        /*min size of subsection for x direction*/
                 int ymin;        /*min size of subsection for y direction*/
                 int xmax;        /*max size of subsection for x direction*/
                 int ymax;        /*max size of subsection for y direction*/
		 int manhatten;   /*-1=unitialized, o.w., 0 or 1*/
                 int unsel;       /*number of points not yet selected*/
		 int id;          /*Id of polygon, 0 if no id*/
		 int xgeom_id;	  /*xgeom internal id*/
               } Rpoly;

typedef struct { int n;           /*Number of points in polygon*/
		 int max;         /*Max x,y points memory allocated*/
		 int *xy0;        /*List of x,y vertices*/
               } Ipoly;

typedef struct { int n;           /*Number of polygons*/
		 int max;         /*Max num of polygons allocated*/
		 Rpoly *rpoly0;   /*Floating point coord polygons*/
		 Ipoly *ipoly0;   /*Integer coord polygons*/
               } Poly_list;

typedef struct { Rtyp rdc;        /*Ohms / sqr at DC*/
                 Rtyp rrf;        /*Skin effect resistivity*/
                 Rtyp xdc;        /*Skin effect reactance*/
		 char *name;      /*Name of metalization*/
		 int attr;        /*Drawing attribute (e.g. color)*/
               } Met_type;

typedef struct { int x0,x1,        /*x extent, x0<=x1*/
		     y0,y1,        /*y extent, y0<=y1*/
                     poly_index,ipt, /*line segment in rpoly for port*/
		     ilev,         /*level for port*/
		     iside,        /*Side number on box (-1 if not edge)*/
		     nsub;         /*number of subsections in port*/
                 int index;        /*port number*/
                 int xgeom_index;  /*port number know to xgeom/emvu*/
				   /* 2.2e */
                 int sign;         /*used for negative current ports*/
                 Rtyp rp,xp,lp,cp; /*Impedance for normalizing S-param*/
		 int line_nu;	   /* line number in .geo file that    */
				   /* the port is defined on.          */
               } port_typ;

typedef struct { int n;            /*Total number of ports*/
                 int m;            /*n-m ports shorted together*/
		 port_typ *port0;  /*head of port list*/
               } port_list;

/*Valid values for via->typ*/
#define VIA_EDGE 0
#define VIA_OLD 1

typedef struct {int  poly_index,ipt, /*line segment in rpoly for via edge*/
				     /* if ipt = -1, then entier poly    */
				     /* is the via 			 */
		     ilev;           /*level for via			 */
		 int num_center_pts; /* number of points in center_pts 	 */
		 int *center_pts;    /* center point for each via 	 */
		 Ipoly *ipoly;	     /* Ipoly description of the via	 */
				     /* subsectioning 			 */
		 Rtyp xy[4];	     /* endpoints for via in xy-cord.	 */
		 int typ;            /* Old or new edge via.             */
		 int subsection;     /* if true then the via should be	 */
				     /* subsectioned. The present 	 */
				     /* subsectioning may be incorrect	 */
		 int line_nu;	     /* line number in .geo file that    */
				     /* the via is defined on.           */
               } via_typ;

typedef struct { int n;             /*Number of vias*/
		 via_typ *via0;     /*head of via list*/
               } via_list;

/*fundamental em subsection types*/
#define NUM_ESUB 7
#define RFX 0
#define RFY 1
#define ARFX 2
#define ARFY 3
#define LOOP 4
#define CROSS 5
#define VIA 6

/*geometry subsection types*/
#define NUM_GSUB 11
#define GRFX 0
#define GRFY 1
#define GENDXP 2
#define GENDYP 3
#define GENDXN 4
#define GENDYN 5
#define GTPP 6
#define GTPN 7
#define GTNP 8
#define GTNN 9
#define GVIA 10

/*msub types*/
/*First NUM_GSUB must be same as GSUBs above*/
/*First NUM_GSUB are also used as fsub types*/
#define NUM_MSUB 13
#define MRFX 0
#define MRFY 1
#define MENDXP 2
#define MENDYP 3
#define MENDXN 4
#define MENDYN 5
#define MTPP 6  /*MUST stay first of triangles*/
#define MTPN 7
#define MTNP 8
#define MTNN 9  /*MUST stay last of triangles*/
#define MVIA 10
#define MXR 11
#define MYR 12

/*Maximum number of real parameters for sub_typ->dxp*/
#define MAXDXP 3
#define MAXDYP 3
#define MAXDZP 2
/*Maximum number of 2x2 int coord sqrs for a subsection*/
#define MAXSQR 5
/*Outline generation mode*/
#define STAIRCASE_FILL 0
#define TRI_FILL 1
#define COR_FILL 2

typedef struct { Rtyp (*fx)(),(*fy)(),(*fz)();  /*Used to calculate Grx,etc.*/
		 Rtyp *dxp,*dyp,*dzp;   /*parameters (size) for Grx,etc.*/
		 int id;           /*see above #defn's*/
		 int cosm,cosn;    /* T or F */
		 int modd,nodd;    /* T or F */
		 Rtyp weight;      /*multiplies coupling*/
		 int num_sqr;      /*number of 2x2 int coord sqrs in sub*/
		 int *sqr_coord;   /*relative to center, x and y*/
		 int *ej_loc;      /*row,col in Box->ej_loss for met loss integral*/
               } sub_typ;

typedef struct { int ex0,ex1;      /*start, stop esub coords*/
                 int ey0,ey1;      /*ditto for y*/
		 Rtyp *wx,*wy;     /*weight lists*/
	       } wt_info;

/*msub coordinate indices*/
#define MX0 0
#define MY0 1
#define MX1 2
#define MY1 3
#define MX2 4
#define MY2 5

/*The msub allows variable sized subsections (MXR,MYR).
*/
typedef struct { int id;           /*What kind of msub?*/
		 int loc;          /*position in matrix*/
		 int *ip;          /*array of int info*/
	       } msub;

typedef struct { int n;            /*number of msubs*/
		 int max;          /*number of positions allocated*/
		 msub *msub0;      /*head of msub list*/
		 int *mask;        /*What em subs are used?*/
	       } msub_list;

typedef struct { int n;            /*Number of levels, includes ground plane, not top cover*/
		 Poly_list *plev0; /*List of polygons on each level*/
		 port_list *ports; /*list of ptrs to 1st vertice of port*/
		 via_list *vias;   /*list of ptrs to 1st vertice of via*/
		 msub_list *lev0;  /*head of msub list array*/
		 int nloc;         /*Size of system matrix (# of wts)*/
		 int max_wt_len;   /*max len of vec used in do_coup*/
		 wt_info **wt0;    /*List of wieghts for all subsections*/
                 Rtyp *fwt0;       /*Weights for each type of esub/fsub combo*/
               } Cir;

/*Maximum number of box resonance indices to store*/
#define NUM_RES 5

typedef struct { char *file;      /*file name for box*/
		 char license_id[256]; /* License id for customer */
		 char xgeomdate[40]; /* Last date xgeom file was changed */
		 Rtyp A,B,C;      /*Box dimensions along X,Y,Z resp.*/
		 Rtyp *H;         /*substrate thicknesses, top = H[0] */
		 int Capm,Capn;
		 int mat_size;    /*System matrix size*/
		 int quad_prec;   /*Use long double for matrix fill calc*/
		 Rtyp qfmin;      /*Freq MHz below which always use quad_prec*/
		 int Num_levels;  /*Number of metalization levels, >= 1*/
		 int Xsym;        /*Is circuit and exitation symmetric?*/
		 Rtyp *Eps,*Mu;   /*Epsilon and Mu for each layer*/
		 Rtyp *Eloss,*Mloss;  /*Dielectric and mag. loss tangent*/
		 int Lossless;    /*True if there is no dielectric loss*/
		 int met_loss;    /*True if metalization loss*/
		 int met_react;   /*True if metalization reactance*/
		 int num_met;     /*Number of lossy metalizations.*/
		 Met_type *met;   /*One for each metalization.*/
		 Met_type Bot,Top;  /*Bottom and top plate surface Res.*/
		 Rtyp rref[8];    /*Array of four ref. plane offsets*/
		 int ref[8];      /*Array of four ref. plane offsets*/
		 sub_typ *sub;    /*Head of subsection type desription list*/
		 char Lstr[7];    /*String for length units*/
		 Rtyp Lcnv;       /*Conversion constant for length units*/
		 int num_ann;
		 char **ann;      /*Array of circuit annotations*/
		 int num_ej;      /*nxn size of ej array*/
		 Rtyp *ej_loss;   /*Integral e.j for met loss*/
		 int num_zterp;   /*Last num_zterp el in Zhat for interp.*/
		 Rtyp kc_min,kc_max;  /*Limits of zhat interp (kc**2)*/
		 int maxm,maxn;   /*Limits for normal Zhat*/
		 int msub_per;    /*min msubs per wavelength*/
                 Rtyp eeff;       /*if >= 1.0, used to calc wavelength*/
		 int max_xinc;    /*max size of msub in x dir*/
		 int max_yinc;    /*max size of msub in y dir*/
                 int ckp_lev;     /*Flag, ck poly edges on adj lev's*/
                 int rm_reson;    /*Flag=1, remove resonances*/
                 int te_res[3*NUM_RES];  /*TE resonance index storage*/
                 int tm_res[3*NUM_RES];  /*TM resonance index storage*/
		 int orphan;       /*=0 if no orphan metal, =1 o.w.*/
               } Box_info;

/*Default values for Spice model limits*/
#define SPICE_CMIN_DEF 0.1 /*pF*/
#define SPICE_LMAX_DEF 100.0 /*nH*/
#define SPICE_RMAX_DEF 1000.0 /*Ohms*/
#define SPICE_MMIN_DEF 0.01
#define SPICE_RZER_DEF 0.0 /*Ohms*/

typedef struct { char *file;        /*Name of file with analysis parameters*/
                 Mat_ptr cans;      /*Circuit theory result*/
                 Mat_ptr uans;      /*un-de-embedded result*/
                 int jxy;           /*If true, do current distributions*/
                 Mat_ptr jans;      /*Current distribution result*/
		 int Cyc;           /*Number of cycles of summation*/
		 int Mem_saver;     /*If true, use single precision matrix*/
                 int nsubs_only;    /*If true, print nsubs and quit*/
		 int State;         /*Print out current analysis state?*/
		 Rtyp cmin;         /*Minimum capacitance in Spice*/
		 Rtyp lmax;         /*Maximum inductance in Spice*/
		 Rtyp rmax;         /*Maximum resistance in Spice*/
		 Rtyp mmin;         /*Minimum mutual L in Spice*/
		 Rtyp rzero;        /*Ohms to be used when R=0 N.G.*/
		 Rtyp fmax;         /*freq to determine max sub size*/
		 int Num_freq;
		 Rtyp *Freq;        /*Array of analysis frequencies, Hz*/
		 char Funit;        /*G for GHz or M for MHz*/
                 Rtyp *rp,*xp,*lp,*cp;  /*Port normalizing impedances*/
		 char **ann;        /*Annotations from .an file*/
		 int num_ann;
		 int mode_calc_time; /*Time (sec) required for mode calc*/
		 int mat_fill_time;  /*Time (sec) required for matrix fill*/
		 int mat_solve_time; /*Time (sec) required for matrix solve*/
               } An_info;

typedef struct { char *dbed_file;       /*File name for de-embedded data*/
                 char *ndbed_file;      /*File name for un-dembedded data*/
		 char *jxy_file;        /*File name for current dist.*/
		 char *spice_file;      /*File name for spice output*/
                 char *cir_name;        /*Name of circuit for output file*/
		 char Param;            /* z, y or s parameters*/
		 int  dbed;             /*=0, no dbed, =1 dbed*/
		 int  ndbed;            /*=0, no save un-dbed data, etc.*/
                 int  spice;            /* if true, output spice data*/
                 int  spice_prec;       /* Number of digits to right of .*/
		 char Cvend;            /* T-Touchstone output
					   C-Compact
					   H-HP Mantis
					   S-Sarnoff Labs Midas */
		 char Com[20];          /*Delimit comments in output file*/
	       } Fmt_info;

