
/******************************************************************/
/*            Installation program for em and xgeom		  */
/* Copyright (c) 1990 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/

/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: getnode.i
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Get node id. (MACHINE DEPENENT CODE)
	
	CONTENTS:
		getnode()
	
*******************************************************************/

/* definitions	*/

/* external	*/

/* structures and global/semiglobal variable */

/*******************************************************************

	NAME: get_node_id
	
	DESCRIPTION: 
		Returns a point to a string containing the node id
		for the computer.  If none, returns NULL.
	
	RETURNS: none

*******************************************************************/


#ifdef NO_EM_SECURITY

char *
get_node_id()
{
	return("none");
}

#else  /* NO_EM_SECURITY */

		/*******************************
		*                              *
		*             sun              *
		*                              *
		*******************************/

#ifdef SUN

char *
get_node_id()
{
	static char node_id[50];
	unsigned gethostid();
	unsigned hostid;

	hostid = gethostid();
	sprintf(node_id,"%x",hostid);
	return (node_id);
}

		/*******************************
		*                              *
		*             HP 700           *
		*                              *
		*******************************/
#endif

/* HP-9000 Series 700 */

#ifdef HP700

#define _INCLUDE_POSIX_SOURCE
#include <sys/utsname.h>
#include <string.h>
char *
get_node_id()
{
	int rc;
	struct utsname name;
	static char node_id[50];

	rc = uname(&name);
	if ( rc < 0 ) {
		fprintf(stderr,"Could not get machine number.\n");
		perror("");
		return (NULL);
	}
	strncpy(node_id,name.__idnumber,50);
	return(node_id);
}

#endif /* HP700 */

		/*******************************
		*                              *
		*             HP 300           *
		*                              *
		*******************************/

#ifdef HP300

/* HP-9000 Series 300/400 */

/* Needed to use HIL to read security code from security module */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/hilioctl.h>
char *
get_node_id()
{
	static char *node_id;
	unsigned hostid;
	int i;

	int fd;
	unsigned char hildev[11];
	unsigned char describe[11];
	unsigned serial_number[11];
	int got_serial_number;
	long int sr_number;
	int status;
	LOCAL_FN char *get_serial_number();

	got_serial_number = 0;
	for (i=0; i<7; i++) {
		sprintf(hildev,"/dev/hil%d",i+1);
		fd = open( hildev, O_RDONLY );
		if ( fd != -1 ) {
			status = ioctl (fd, HILID, &describe[0]);
			if ( (status != -1) && (describe[0] == 0x34) ) {
				/* There is an HP-HIL device and is an ID Module */
				node_id = get_serial_number(fd);
				got_serial_number = -1;
			}
			close (fd);
		}
	}
	if ( ! got_serial_number) {
		fprintf(stderr, "Could not find ID module\n");
		return(NULL);
	} else {
		return (node_id);
	}
}

LOCAL_FN char *
get_serial_number(hil_fd)
int hil_fd;		/* file descriptor */
{
	int i,len;
	int status;
	long int sr_number;
	static char node_id[50];
	unsigned char country;
	union srnum {
		unsigned long int number;
		struct {
			unsigned dummy :2;
			unsigned byte8 :6;
			unsigned byte7 :8;
			unsigned byte6 :8;
			unsigned byte5 :8;
		} r;
	} s;

	union hil_sc {
		unsigned char describe[9];
		struct {
			unsigned format       :4;
			unsigned undefined    :4;
			unsigned bytes2       :8;
			unsigned bytes3       :8;
			unsigned bit17        :1;
			unsigned product_letter     :7;
			unsigned sr_bytes5    :8;
			unsigned sr_bytes6    :8;
			unsigned sr_bytes7    :8;
			unsigned res1         :2;
			unsigned sr_bytes8    :6;
			unsigned res2         :1;
			unsigned country      :7;
		} sc;
	} hilsc;

	/* Use Report Security Code command to get ID Module information */
	status = ioctl(hil_fd, HILSC, &hilsc.describe[0]);

	/* Put bits and bytes in proper order to make valid serial Number */
	s.r.byte5 = hilsc.sc.sr_bytes5;
	s.r.byte7 = hilsc.sc.sr_bytes7;
	s.r.byte6 = hilsc.sc.sr_bytes6;
	s.r.byte8 = hilsc.sc.sr_bytes8;

	/* get country */
	country = hilsc.sc.country; 

	/* return serial number as long int value */
	sprintf(node_id,"%d",s.number);
	len = strlen(node_id);
	for (i=len+1; i>4; --i) {
		node_id[i] = node_id[i-1];
	}
	node_id[4] = country;
	return ( node_id);
}

#endif

		/*******************************
		*                              *
		*             Appolo           *
		*                              *
		*******************************/

#ifdef APOLLO

/* Apollo DN-3500..., DN-10000 */

#include "/usr/apollo/include/builtins.h"

char *
get_node_id()
{
	static char node_id[50];
	/* 
	** Def global incase time_out does both time and node
	** protection, def of uid cannot be in middle of routine
	*/
	long uid[10];
	proc2_$who_am_i( uid );
	uid[1] &= 0x000fffff;
	sprintf(node_id,"%x",uid[1]);
	return (node_id);
}

#endif

/* System specific include file.  krs 31Aug90 */


		/*******************************
		*                              *
		*             Ibm6000          *
		*                              *
		*******************************/
/*
****
****		MAN PAGE FOR UNAME
****

  uname or unamex Subroutine
      
  
     Key Concepts
  
      
  
    Purpose
  
      Gets the name of the current AIX system.
  
    Library
  
      Standard C Library (libc.a)
  
    Syntax
  
  #include <sys/utsname.h>
  
  int uname (Name)
  struct utsname *Name;
  
  int unamex (Name)
  struct xutsname *Name;
  
    Description
  
      The uname subroutine stores information identifying the
  current system in the structure pointed to by the Name parameter.
  
      The uname subroutine uses the utsname structure, which is de-
  fined in the sys/utsname.h file, and it contains the following
  members:
  
  char	sysname[SYS_NMLN];
  char	nodename[SYS_NMLN];
  char	release[SYS_NMLN];
  char	version[SYS_NMLN];
  char	machine[SYS_NMLN];
  
      The uname subroutine returns a null-terminated character
  string naming the current system in the sysname character array.
  The nodename array contains the name  that the system is known by
  on a communications network.  The release and version arrays
  further identify the system.  The machine array identifies the
  system unit hardware being used.
  
      The unamex subroutine uses the xutsname structure, which is
  defined in the sys/utsname.h file, and it contains the following
  members:
  
  unsigned long	nid;
  long	reserved[3];
  
      The xutsname.nid field is the binary form of the
  utsname.machine field.  For local area networks in which
  a binary node name is appropriate, xutsname.nid contains such a
  name.
  
    Parameter
  
       Name	A pointer to the utsname or xutsname structure.
  
    Return Values
  
      Upon successful completion, the uname or unamex subroutines
  return a non-negative  value.    Otherwise,  a value of -1 is re-
  turned and the global variable errno is set to indicate the er-
  ror.
  
    Error Code
  
      The uname and unamex subroutines fail if the following is
  true:
  
      EFAULT	The Name parameter points outside of the process ad-
  dress space.
  
    Implementation Specifics
  
      These subroutines are part of AIX Base Operating System (BOS)
  Runtime.
  
    Related Information
  
      The  uname command.
*/

#ifdef IBM6000

#include <sys/utsname.h>

char *
get_node_id()
{
	static char node_id[50];
	struct utsname conf;
	int rc;
	rc = uname (&conf);
	if ( rc == -1 ) {
		fprintf(stderr,"Could not get machine number.\n");
		perror("");
		return (NULL);
	}
	strncpy(node_id,conf.machine,50);
	return (node_id);
}

#endif

		/*******************************
		*                              *
		*             DOS              *
		*                              *
		*******************************/

#ifdef DOS

char *
get_node_id()
{
	static char node_id[50];
	static int first_time_called = TRUE;
	char * hostid;
	extern char *gethostid();

	if ( first_time_called ) {
		first_time_called = FALSE;
		hostid = gethostid();
		if ( hostid == NULL ) {
			return(NULL);
		}
		strcpy(node_id,hostid);
	}
	return (node_id);
}

#endif

		/*******************************
		*                              *
		*          DEC 5000            *
		*                              *
		*******************************/

#ifdef DEC5000

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>

#define ETHER_DEVICE		"ln0"
#define OLD_ETHER_DEVICE	"se0"

char *
get_node_id()
{
	static char node_id[50];

	struct ifdevea devea;	/* device info used/returned by ioctl() */
	int nisocket;	/* socket id*/
	int i;

	/*
	**	Connect to socket
	*/
	if (( nisocket = socket (AF_INET, SOCK_DGRAM, 0) ) < 0 ) {
		perror("socket");
		return(NULL);
	}

	/*
	**	Fill in the device struct. The default_pa field
	**	is the ethernetaddress.  "ln0" is the new device driver
	**	name used begining in release 4.0.
	*/

	strcpy(&devea.ifr_name[0], ETHER_DEVICE);
	if ( ioctl( nisocket, SIOCRPHYSADDR, &devea) >= 0 ) {
		sprintf(node_id,"%02x%02x%02x%02x%02x%02x",
			devea.default_pa[0],
			devea.default_pa[1],
			devea.default_pa[2],
			devea.default_pa[3],
			devea.default_pa[4],
			devea.default_pa[5]);
		return ( node_id );
	}

	/*
	**	Fill in the device struct. The default_pa field
	**	is the ethernetaddress.  "se0" is the old device driver
	**	name used before release 4.0.
	*/

	strcpy(&devea.ifr_name[0], OLD_ETHER_DEVICE);
	if ( ioctl( nisocket, SIOCRPHYSADDR, &devea) >= 0 ) {
		sprintf(node_id,"%02x%02x%02x%02x%02x%02x",
			devea.default_pa[0],
			devea.default_pa[1],
			devea.default_pa[2],
			devea.default_pa[3],
			devea.default_pa[4],
			devea.default_pa[5]);
		return ( node_id );
	}

	/*
	**	Both ioctl failed.  Run them again to get the error messages.
	*/
	strcpy(&devea.ifr_name[0], ETHER_DEVICE);
	if ( ioctl( nisocket, SIOCRPHYSADDR, &devea) < 0 ) {
		perror(ETHER_DEVICE);
	}

	strcpy(&devea.ifr_name[0], OLD_ETHER_DEVICE);
	if ( ioctl( nisocket, SIOCRPHYSADDR, &devea) < 0 ) {
		perror(OLD_ETHER_DEVICE);
	}

	fprintf(stderr, "Could not get ethernet address\n");
	return(NULL);
}

#endif

#endif /* NO_EM_SECURITY */
