/******************************************************************/
/*            	Unlock program for em and xgeom		  	  */
/* Copyright (c) 1990 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/

/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: wkeyfile.i
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Converts key into a single string
	
	CONTENTS:
		write_key_string() - write key to a string

 	INTERNAL:
		write_prog_string() - write all value to the keyfile for 
				      a specific program
		append_str() - appends single keyfile type line to string
		node_compare() - compares two node ids.
	
*******************************************************************/

/* definitions  */
#undef TAB
#define TAB     '\t'

/* external	*/

/* structures and global/semiglobal variable */

LOCAL_FN void
write_prog_string(
#if NeedFunctionPrototypes
	char **string,
	Prog *prog,
	int making_key
#endif
);

LOCAL_FN void
str_append(
#if NeedFunctionPrototypes
	char **string,
	char *progname,
	char *action,
	char *value
#endif
);

int
node_compare();

/*******************************************************************
	NAME: write_key_string
	
	DESCRIPTION: 
		Creates a key string containing the key.
		This includes newlines so string can be printed.
		to a file.  The string is an character undefined 
		pointer. It includes all items but key value.

	RETURNS: none

*******************************************************************/
void
write_key_string(string,key,making_key)
char **string;
Key *key;
int making_key;	/* if TRUE, string will be used to make a key 	    */
		/* if FASLE string is being used to write to a file */
{
	int i;
	
	*string = (char *) new( sizeof(char) );
	if (*string == NULL) {
		fprintf(stderr,"Could not allocated memory to store string\n");
		exit (1);
	}
	*string[0] = '\0';
	
	str_append(string,"release","cust",key->release);
	str_append(string,"license","cust",key->license);
	write_prog_string(string, &key->em,    making_key);
	write_prog_string(string, &key->xgeom, making_key);
	write_prog_string(string, &key->emvu,  making_key);
	write_prog_string(string, &key->patvu, making_key);
	for ( i=0; i< key->nu_sn; ++i ) {
		str_append(string,"sn","cust",key->sn[i]);
	}
	if ( making_key ) {
		tolowerstr(*string);
	}
}


/*******************************************************************
	NAME: write_prog_string
	
	DESCRIPTION: 
		Append out all parameters for program to string.
	
	RETURNS: none

*******************************************************************/

LOCAL_FN void
write_prog_string(string,prog,making_key)	
char **string;
Prog *prog;
int making_key;	/* if TRUE, string will be used to make a key 	    */
		/* if FASLE string is being used to write to a file */
{
	char date[DATE_LEN];
	char value_str[20];
	int i;
	void convert_to_date();

	if ( prog->have_prog == TRUE ) {
		if ( prog->alike == ALIKE_EM ) {
			str_append(string,prog->name,"lock","em");
		}

		if ( prog->expir_date == TRUE ) {
			convert_to_date(date,&prog->expir);
			str_append(string,prog->name,"date",date);
		}

		if ( prog->nu_nodes != NONE ) {
			/*
			** Sort the program names, if string is ment for a key
			*/
			if ( making_key && prog->nu_nodes > 1 ) {
				qsort( (char *)prog->nodes, (size_t)prog->nu_nodes,
				       sizeof(char *),
				       node_compare );
			}

			for ( i=0; i<prog->nu_nodes; ++i) {
				str_append(string,prog->name,"node",prog->nodes[i]);
			}
		}
		if ( (prog->subs != NONE) && (prog->subs != 0) ) {
			sprintf(value_str,"%d",prog->subs);
			str_append(string,prog->name,"subs",value_str);
		}
	}
}


/*******************************************************************
	NAME: str_append
	
	DESCRIPTION: 
		Appends to the sting the following
			progname /t action /t value /n 

	RETURNS: none

*******************************************************************/
LOCAL_FN void
str_append(string,progname,action,value)
char **string;
char *progname;
char *action;
char *value;
{
	unsigned len;	
	len = strlen(*string) 
	      + strlen(progname)
	      + strlen(action)
	      + strlen(value)
	      + 3   /* for tabs and newline */
	      + 1;  /* for end of line null character */
	      
	*string = (char *)rnew(*string,(unsigned)len);

	strcat(*string,progname);
	strcat(*string,"\t");
	strcat(*string,action);
	strcat(*string,"\t");
	strcat(*string,value);
	strcat(*string,"\n");
}

/*******************************************************************
	NAME: node_compare (INTERNAL)
	
	DESCRIPTION: 
		compares two node id. Used by qsort.
	
	RETURNS: 
		<0 - node1 < node2
		0 -  node1 = node2
		>0 - node1 > node2

*******************************************************************/

int
node_compare(node1,node2)
char **node1, **node2;
{
	char str1[100];
	char str2[100];

	strncpy(str1,*node1,99);
	strncpy(str2,*node2,99);
	tolowerstr(str1);
	tolowerstr(str2);
	return ( strcmp(str1,str2) );
}
