/* Data definitions for jxy calculations */ /*10apr91jcr*/

/*Indices for changing source information*/
#define NEW_VM 0
#define NEW_PH 1
#define NEW_R 2
#define NEW_X 3
#define NEW_L 4
#define NEW_C 5

/*Indices for rect information*/
#define RX0 0
#define RY0 1
#define RX1 2
#define RY1 3
#define RXA 4
#define RXB 5
#define RYA 6
#define RYB 7
#define RSIZE 8

/*Indices for triangle information*/
#define TX0 0
#define TY0 1
#define TX1 2
#define TY1 3
#define TX2 4
#define TY2 5
#define TA0 6
#define TSIZE 7

/*Mask values for which data to be calculated*/
#define CALC_JX 1
#define CALC_JY 2
#define CALC_EZ 4  /*Actually calculates charge density*/

/*Values for which data is to be steped (pdat->pstep)*/
#define STEP_F 1
#define STEP_VM 2
#define STEP_VP 3
#define STEP_R 4
#define STEP_X 5
#define STEP_L 6
#define STEP_C 7

/*Indices of statisitics for calculated data*/
#define JX_MG_MAX 0
#define JX_MG_MIN 1
#define JX_MG_AVE 2
#define JX_MG_SIG 3
#define JX_PH_MAX 4
#define JX_PH_MIN 5
#define JX_PH_AVE 6
#define JX_PH_SIG 7
#define JY_MG_MAX 8
#define JY_MG_MIN 9
#define JY_MG_AVE 10
#define JY_MG_SIG 11
#define JY_PH_MAX 12
#define JY_PH_MIN 13
#define JY_PH_AVE 14
#define JY_PH_SIG 15
#define EZ_MG_MAX 16
#define EZ_MG_MIN 17
#define EZ_MG_AVE 18
#define EZ_MG_SIG 19
#define EZ_PH_MAX 20
#define EZ_PH_MIN 21
#define EZ_PH_AVE 22
#define EZ_PH_SIG 23
#define NUM_STATS 24


/*Structure to store recangle data to be ploted*/
typedef struct Pdats { 
               Rtyp x0,y0;      /*Start coord of plot area relative to zoom area*/
               Rtyp xx0,yy0;    /*Start coord of zoom area on substrate*/
               int nx,ny;       /*Number of points in x,y array*/
               Rtyp dx,dy;      /*Step size*/
               int *ira,*irb;   /*ptr to start & stop rects*/
               int pvar;        /*Parameter to be steped*/
               int pport;       /*Some parameters need a port number*/
               Rtyp *plist;     /*List of values of param for calc.*/
               int pn;          /*Number of steps for param.*/
               float *jx_mg;    /*magnitude of Jx*/
               float *jx_ph;    /*phase (-pi to pi)of Jx*/
               float *jy_mg;    /*magnitude of Jy*/
               float *jy_ph;    /*phase (-pi to pi)of Jy*/
               float *ez_mg;    /*magnitude of Ez*/
               float *ez_ph;    /*phase (-pi to pi)of Ez*/
               Rtyp *stats;     /*Max/min,ave,sigma range limit of plot data*/
                                /*allocated only for head of pdat list*/
               struct Pdats *next; /*Next pdat structure*/
             } Pdat;

/*Structure to store triangle data to be plotted*/
typedef struct Tdats { 
               Ipts *tri;       /*list of triangle coordinates*/
               Rtyp x0,y0;      /*Start coord of plot area relative to zoom area*/
               int pn;          /*Number of steps for param.*/
	       int snapped;     /*Vertice cooord conv. to pixel & snapped?*/
               Rtyp dx,dy;      /*Step size*/
               float *jx_mg;    /*magnitude of Jx, pn pts for each triangle*/
               float *jx_ph;    /*phase (-pi to pi)of Jx*/
               float *jy_mg;    /*magnitude of Jy*/
               float *jy_ph;    /*phase (-pi to pi)of Jy*/
               float *ez_mg;    /*magnitude of Ez*/
               float *ez_ph;    /*phase (-pi to pi)of Ez*/
             } Tdat;

typedef struct Fdats { int nport;       /*Number of ports*/
               int nsub;        /*Number of subsections*/
               Rtyp freq;       /*Frequency(MHz)*/
               Rtyp *Eeff;      /*Epsilon effective, one per port*/
               Rtyp *Z0;        /*Characteristic impedance...*/
               Mat_ptr cans;    /*Circuit theory S-parameters*/
               Mat_ptr ypr;     /*Current data from em*/
               Vec_ptr jpr;     /*Actual current given src*/
               struct Fdats *next; /*Data at next frequency*/
             } Fdat;

typedef struct Jdats { Box_info *box;   /*Box paramters*/
               Cir *cir;        /*Circuit geometry, subsections*/
               char *cmd;       /*Cmnd line used to run em*/
               char *nam;       /*Name of .geo file used*/
               char *tim;       /*Time/date string em was run*/
               char *ver;       /*Em version information*/
               int nport;       /*Number of ports*/
               int nsub;        /*Number of subsections*/
               Rtyp *vsrc_mg;   /*Voltage mag.*/
               Rtyp *vsrc_ph;   /*Voltage phase, radians*/
               Rtyp *rsrc;      /*Source input resistance*/
               Rtyp *xsrc;      /*Source input reactance*/
               Rtyp *lsrc;      /*Source input inductance (Henries)*/
               Rtyp *csrc;      /*Source input capacitance (Farads)*/
               Ipts *rect;      /*List of rectangles*/
               int *xterp;      /*X coord of nth msub for interpolation*/
               int *yterp;      /*Y coord of nth msub for interpolation*/
               int nfreq;       /*Number of frequencies*/
               Rtyp *freq;      /*Frequency list*/
               Fdat *fdat;      /*Ptrs to fdat structures*/
               struct Jdats *next; /*Next jdat structure*/
             } Jdat;
