#ifndef __keyfile_h
#define __keyfile_h

#define DEFAULT_KEYFILE_DIR	"/home/sonnet"

#define DEFAULT_KEYFILE_NAME	"sonnet.key"	/* must be less than 14 chars*/

#define	FILE_LEN		512
#define	KEY_LENGTH		30
#define	RELEASE_LEN		10

#define		ALIKE_EM	1
#define		ALIKE_UNIQ	2

#define		KEY_GOOD		1
#define		KEY_BAD			2
#define		KEY_WRONG_CHECKSUM	3

#define	QUESTION_LEN		256
#define	LICENSE_LEN		256
#define DATE_LEN		15	/* character length of date */

#define AUX_ERR_MSG		"AUXILARY ERROR MESSAGE: "

#define DO_NOT_PRINT_ERROR      1
#define PRINT_ERROR             2
	
#define	NONE	-1
#define	SAME	-2

#define		PASSWORD_UNDEFINED	1
#define		PASSWORD_ALPHA		2
#define		PASSWORD_NUMERIC	3

#define KEY_NOT_DEFINED "Not Defined"

typedef struct
{
	int day;		/* Date 1-31 		*/
	int month;		/* Month 0-11 		*/
	int year;		/* Year 1900- 		*/
} Date;

typedef struct
{
	int have_prog;		/* TRUE - have a copy of the prog, else    */
				/* FALSE    				   */
	int alike;		/* indicates if lock is the same as 	*/
				/* other programs				*/
				/* ALIKE_EM - lock same as EM		*/
				/* ALIKE_UNIQ - has a uniq lock		*/
	char *name;		/* name of the program			   */
	int expir_date;      	/* if have expiration date, TRUE, else FALSE*/
	Date expir;		/* expiration date			   */
	int nu_nodes;		/* number of nodes, contains NONE if none  */
	char **nodes;		/* character string of node ids 	   */
	int subs;		/* Limit on nu. of subsections may be NONE */
				/* Implemented only for em		   */
} Prog;

typedef struct {
	char license[LICENSE_LEN]; /* License id for customer.  There is   */
				   /* one license id per keyfile in the    */
				   /* field.				   */
	int file;		/* TRUE if there is a key file, else FALSE */
	char filename[FILE_LEN];	/* name of key file */
	char release[RELEASE_LEN];	/* Release of software. */
	int nu_sn;		/* number of serial number, contains NONE  */
				/* if none				   */
	char **sn;		/* character string of serial numbers 	   */
	Prog em;
	Prog xgeom;
	Prog emvu;
	Prog patvu;
	char key[KEY_LENGTH];
} Key;

#endif /* __keyfile_h */
