/* header file for WPatData */

/* constants */
#define    PI       3.141592653589793238462643
#define    TWO_PI         6.283185307179586476925287
#define PTN_IMPEDANCE 377.0
#define RAD_2_DEG   57.295779513082320876798155
#define DEG_2_RAD   (PI/180.0)
#define EPS         1e-06
#define EPS_FREQ    1e-03  

#define FREQ_MHZ 0
#define FREQ_GHZ 1
#define ANG_RAD    0
#define ANG_DEGREE 1

#define YES  1
#define NO  -1

/* Approximation level for computing patterns */
#define PTN_APPRX_DIPOLE                6
#define PTN_APPRX_NO_DIELECTRICS	7
#define PTN_APPRX_RECIPROCITY           8

/*Indices for various coordinates system */
#define PTN_COORD_SPHERICAL  0
#define PTN_COORD_POLAR      1
#define PTN_COORD_EARTH      2
#define PTN_COORD_ARRAY      3
#define PTN_COORD_ARRAY_SINE 4

/*Indices for components to be viewed */
#define PTN_VIEW_TOTAL      0
#define PTN_VIEW_THETA      1
#define PTN_VIEW_PHI        2

/*Indices for scale */
#define PTN_SCALE_LOG       0
#define PTN_SCALE_LINEAR    1

typedef struct {
	double theta,phi;  /* Two angle coordinates */
	double sin_t,cos_t; /* sine and cosine of theta */
	double sin_p,cos_p; /* sine and cosine of phi */
	int coord;        /* coordinate system used */
} Angles; 

typedef struct {
	double sin_t,cos_t; /* Sine and cosine of transmitted angle */
        double eta_r;       /* ratio of impedance to the impedance in
                             the top dielectric (air) */
	double beta_r;      /* ratio of phase velocity */
	double h_r;         /*  normalized thickness */
	double kz;          /* z-coordinate relative to the top dielectric */
	double constant;    /* normalization constant, should removed */
	complex factor[2];   /* approximation factor for each layer */
	double rt_10,rt_12, tt_01; /* theta: transmission and reflection 
				      coefficients */
	double rp_10,rp_12, tp_01; /* phi: transmission and reflection
				      coefficients */
	int total_reflection; /* whether total reflection occurs */
} LevelParameters;

typedef struct {
	int argc;          /* number of command line items */
	int verbose;
	int precision;    /* use single or double precision */
	int state_version; /* print out verion and id */
	int state_mem;    /* estimate amount of memory used */
	int state_freq;   /* print a list of frequency in jxy file */
	int state_port;   /* print a list of ports */
	int quick_2d;     /* do a quick 2d */
	int quick_3d;     /* do a quick 3d */
	int debug;        /* debug options */
	int apprx;        /* approximation used, debug option */
	int nray;
	int print_j;      /* print out current, debug option */
	char *confile;    /* antenna configuration file */
	char *jxyfile;    /* current file */
	char *patfile;    /* antenna output file */
	char **argv;    /* command line */
} WPatGenOpt;	 	

typedef struct tagWPatRange {
	unsigned int unit; /* 1st bit: unit
        		      2nd bit: start is set.
			      3rd bit: stop is set.
			      4th bit: step is set.
			    */	
	float start,stop,step;
	struct tagWPatRange *next;
} WPatRange;

typedef struct {
	FILE *confptr;  /* file pointer to confile */
	FILE *patfptr;  /* file pointer to patfile */
	char *version; /* whether Version specified */
	char *time_start; /* starting time */
	char *time_cur;    /* current time */
	char *license_id;
	char *host_id;
	Jdat *jd;
	int header_in; /* whether CTLPG specified */
	WPatRange *freq;
	WPatRange ang1,ang2;
	int icoord;         /* coordinates */
	int iground;        /* Is ground plane present */
	int apprx_level;    /* approximation level */
	int nbyte;          /* MB required */
	int npt;            /* # of data points */
	int *freq_indx;   
	int *port_indx;
} WPatGenTmp;

typedef struct {
	int ipatvu;	/* pattern viewing coordinate system */
	int nfreq;
	float *freq;	/* this would handle non-uniform freq sweeping */
	int ntheta;
	float *theta;	/* same theta range for all freq. */
	int nphi;
	float *phi;	/* same phi range for all freq */
	int nfield;	/* nfreq*nalpha*nbeta*4 */
	float *field;
	char FreqUnit[4];
	int npara;     /* number parameters per freq */
	float *para;	/* para list */
} WPatData;

#define Bit1(x)  ( (x)       & 1)
#define Bit2(x)  (((x) >> 1) & 1)
#define Bit3(x)  (((x) >> 2) & 1)
#define Bit4(x)  (((x) >> 3) & 1)
