#include "local.h"
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "son.h"
#include "em.typ"
#include "jxy.typ"
#include "complex2.h"
#include "patdata.h"

#define ALL -2

/*
 * void WPatGenFile_read(opt,td,ad)
 *   read the antenna configuration file
 *
 */
void WPatGenFile_read(opt,td,ad)
     WPatGenOpt *opt;
     WPatGenTmp *td;
     WPatData *ad;
{
  extern FILE *WFile_open();
  extern char *WFile_ext();
  extern Jdat *read_jdat();
  int i;
  int portindx;
  char *back_file;

  td->confptr=WFile_open(opt->confile,"r");
  if (opt->verbose == YES) 
    printf("Processing %s. \n",opt->jxyfile);
  td->patfptr=WFile_open(opt->jxyfile,"r");
  fclose(td->patfptr);
  td->jd=read_jdat(opt->jxyfile);
  if (td->jd == NULL) exit(1);
  sort_port_indx(td);
  if (opt->state_freq == YES || opt->state_port == YES) {
    if (opt->state_freq == YES) {
      printf("#%10s: Frequency List in %s in file %s:\n",opt->argv[0],opt->jxyfile);
      printf("%11s  GHZ\n"," ");
      for (i=0; i < td->jd->nfreq; i++) 
	printf("%11s  FRE %g\n"," ",*(td->jd->freq+i)/1e+03);
    }
    if (opt->state_port == YES) {
      printf("#%10s: Total number of ports: %d \n",opt->argv[0],td->jd->nport);
      for (i=0; i < td->jd->nport; i++) {
	printf("Port %2d R=%g X=%g L=%g C=%g\n",
	       td->port_indx[i],
	       td->jd->rsrc[i],td->jd->xsrc[i],
	       td->jd->lsrc[i]*1e+9,td->jd->csrc[i]*1e+12);
      }
      printf("#Units: R=ohms,X=ohms,L=nH,C=pF.\n");
    }
    exit(0);
  }

  /* process confile */
  process_confile(opt,td,ad);

  /* check if patfile exists. If so, rename to *.pat */
  td->patfptr=fopen(opt->patfile,"r");
  if (td->patfptr != NULL) {
    fclose(td->patfptr);
    back_file=WFile_ext(opt->patfile,"pab");
    if (rename(opt->patfile,back_file)) 
      fprintf(stderr,"%s: cannot save %s to %s.\n",opt->patfile,back_file);
    sFree((char *)back_file);
  }
  td->patfptr=WFile_open(opt->patfile,"w");
}  
/*
 *PosSubVecInVec(double *v1, int nv1, double *v, int nv, int *indx)
 *  find out the position of sub-vector in vector
 *
 */
void PosSubVecInVec(v1,nv1,v,nv,indx)
     double *v1;
     int nv1;
     double *v;
     int nv;
     int *indx;
{
  int i,ipos;

  for (i=0; i < nv1; i++) {
    ipos=IsDValueInVec(v1[i],v,nv);
    if (ipos != NO) indx[ipos]=ipos;
  }
}

/*
 * float *Range2FVector(WPatRange *pf, int *nvec);
 *  Convert the range data to a sequence of vector 
 *
 */
float  *Range2FVector(pf,nvec)
     WPatRange *pf;
     int *nvec;
{
  float *vf;
  int i;
  
  if (fabs(pf->step) < EPS) *nvec=1;
  else                      *nvec=(pf->stop-pf->start)/pf->step+1.1;
  vf=(float *)new(sizeof(float)*(*nvec));
  for (i=0; i < *nvec; i++)     vf[i]=pf->start+pf->step*i;
  
  return((float *)vf);
}

/*
 * double *Range2DVector(WPatRange *pf, int *nvec);
 *  Convert the range data to a sequence of vector 
 *
 */
double   *Range2DVector(pf,nvec)
     WPatRange *pf;
     int *nvec;
{
  double *vf;
  int i;
  
  if (fabs(pf->step) < EPS) *nvec=1;
  else                      *nvec=(pf->stop-pf->start)/pf->step+1.1;
  vf=(double *)new(sizeof(double)*(*nvec));
  for (i=0; i < *nvec; i++)     vf[i]=pf->start+pf->step*i;
  
  return((double *)vf);
}

static char  *keyword[]={
    "CTLPG",
    "VER",
    "VERSION",
    "COORD",
    "COORDINATES",
    "ANG1",
    "ANGLE1",
    "ANG2",
    "ANGLE2",
    "GHZ",
    "MHZ",
    "FRE",
    "FREQUENCY",
    "PORT",
    "PORTS",
    NULL
}; 

process_confile(opt,td,ad)
     WPatGenOpt *opt;
     WPatGenTmp *td;
     WPatData *ad;
{
  extern int WStr_extract(/* char *in, *out */);
  extern char *WStr_copy(/* char *str */);
  extern char *WTime_local(/**/);
  extern char *get_version();
  extern char *get_node_id();
  extern char *get_license_id();
  WPatRange *SetAllFreq(/* WPatGenTmp *td, WPatRange *pf */);
  WPatRange *SetFirstFreq(/* WPatGenTmp *td, WPatRange *pf */);
  char *s1,*s2,*s3;
  char *p;
  char *plist,*list;
  int nlist,i,istatus,punit;
  unsigned int lineno;
  unsigned int itemno;
  WPatRange *pf,*pf0;
  int port_INDX,iport,iport_mag,iport_phase;
  float value;

  td->header_in=NO;
  td->icoord=NO;
  td->ang1.unit=ANG_DEGREE;
  td->ang2.unit=ANG_DEGREE;
  td->apprx_level=opt->apprx;
  td->version=get_version();
  td->time_start=(char *)new(80*sizeof(char));
  WTime_local(td->time_start);
  td->time_cur= (char *)new(80*sizeof(char));
  td->license_id=get_license_id();
  td->host_id=get_node_id();

  s1=(char *)new(256*sizeof(char));
  s2=(char *)new(256*sizeof(char));
  s3=(char *)new(256*sizeof(char));
  list=(char *)new(256*sizeof(char));
    
  lineno=0;
  itemno=0;
  iport_mag=NO;
  iport_phase=NO;
  pf=td->freq=(WPatRange *)new(sizeof(WPatRange));
  pf->unit=FREQ_MHZ;
  pf0=NULL;
  while (fgets(s1,256,td->confptr) != NULL) {
    lineno++;
    p=s2;
    if (WStr_extract(s1,p) == 1) {
      plist=list;
      nlist=WStr_seperate(p,",= \t",plist);
      for (i=0; keyword[i] != NULL; i++)  {
	if (strcasecmp(plist,keyword[i]) == 0) {
	  itemno++;
	  if (i != 0 && td->header_in == NO)
	      print_error(opt->confile,lineno,plist,"Keyword CTLPG is missing in the first non-commented line");
	  switch (i) {
	  case 0:  /* CTLPG */
	    if (itemno != 1) 
	      print_error(opt->confile,lineno,plist,"Multiple CTLPG specification is not allowed");
	    if (nlist == 1) 
	      td->header_in = YES;
	    else 
	      print_error(opt->confile,lineno,plist);
	    break;
	  case 1:  /* VER */
	  case 2: /* VERSION */
	    if (nlist != 2) 
	      print_error(opt->confile,lineno,plist,
			  "Unkown version is specified");
	    else {
	      p=plist+strlen(plist)+1;
	      if (CheckVersion(td->version,p) == NO)
		print_error(opt->confile,lineno,p,"Invalid Version specified");
	    }
	    break;
	  case 3: /* COORD */
	  case 4: /* COORDINATES */
	    if (td->icoord != NO) 
	      print_error(opt->confile,lineno,plist,"Can only specify coordinate system once.");
	    if (nlist != 2 )
	      print_error(opt->confile,lineno,plist, "Unknown coordinate system specified");
	    else {
	      p=plist+strlen(plist)+1;
	      CheckCoordName(p,td);
	      if (td->icoord == NO) 
		print_error(opt->confile,lineno,p, "Unknown coordinate system specified");	    
	    }
	    break;
	  case 5: /* ANG1 */
	  case 6: /* ANGLE1 */
	    if (td->icoord == NO) 
	      print_error(opt->confile,lineno,plist,"Specify coordinate system before angles");
	    if (Bit2(td->ang1.unit)) 
	      print_error(opt->confile,lineno,plist,"Can only specify angle range once.");
	    if (nlist < 2 || nlist > 4) 
	      print_error(opt->confile,lineno,plist, "Invalid ANG1 range specified");
	    else {
	      p=plist+strlen(plist)+1;
	      td->ang1.start=atof(p);
	      td->ang1.unit += 2;
	      if (nlist >= 3) {
		p=p+strlen(p)+1;
		td->ang1.stop=atof(p);
		td->ang1.unit += 4;
	      }
	      if (nlist == 4) {
		p=p+strlen(p)+1;
		td->ang1.step=atof(p);
		td->ang1.unit += 8;
	      }
	      CheckAngleRange(opt->confile,lineno,td,0);
	    }
	    break;
	  case 7: /* ANG2 */
	  case 8: /* ANGLE2 */
	    if (td->icoord == NO) 
	      print_error(opt->confile,lineno,plist,"Specify coordinate system before angles");
	    if (Bit2(td->ang2.unit))
	      print_error(opt->confile,lineno,plist,"Can only specify angle range once.");
	    if (nlist < 2 || nlist > 4) 
	      print_error(opt->confile,lineno,plist, "Invalid ANG2 range specified");
	    else {
	      p=plist+strlen(plist)+1;
	      td->ang2.start=atof(p);
	      td->ang2.unit += 2;
	      if (nlist >= 3) {
		p=p+strlen(p)+1;
		td->ang2.stop=atof(p);
		td->ang2.unit += 4;
	      }
	      if (nlist == 4) {
		p=p+strlen(p)+1;
		td->ang2.step=atof(p);
		td->ang2.unit += 8;
	      }
	      CheckAngleRange(opt->confile,lineno,td,1);
	    }
	    break;
	  case 9: /* GHz */
	    if (nlist != 1)
	      print_error(opt->confile,lineno,plist+strlen(plist), 
			  "Unknown keyword specified");
	    else  
	      pf->unit=FREQ_GHZ;
	    punit=pf->unit;
	    break;
	  case 10: /* MHz */
	    if (nlist != 1)
	      print_error(opt->confile,lineno,plist+strlen(plist), 
			  "Unknown keyword specified");
	    else 
	      pf->unit=FREQ_MHZ;
	    punit=pf->unit;
	    break;
	  case 11: /* Fre */
	  case 12: /* Frequency */
	    if (nlist < 2 || nlist > 4) 
	      print_error(opt->confile,lineno,plist,
			  "Unknown frequency range specified");
	    else {
	      if (nlist >= 2) {
		p=plist+strlen(plist)+1;
		if (strcasecmp(p,"ALL") == 0) {
		  if (nlist > 2) 
		    print_error(opt->confile,lineno,plist,
				"Unknown frequency range specified");
		  pf0=SetAllFreq(td,pf);
		  pf=pf0;
		}
		else if (strcasecmp(p,"FIRST") == 0) {
		  if (nlist > 2) 
		    print_error(opt->confile,lineno,plist,
				"Unknown frequency range specified");
		  pf0=SetFirstFreq(td,pf);
		  pf=pf0;
		}
		else {
		  pf->start=atof(p);
		  pf->unit += 2;
		}
	      }
	      if (nlist >= 3) {
		p=p+strlen(p)+1;
		pf->stop=atof(p);
		pf->unit += 4;
	      }
	      if (nlist == 4) {
		p=p+strlen(p)+1;
		pf->step=atof(p);
		pf->unit += 8;
	      }
	      CheckFreqRange(opt->confile,lineno,plist,pf,td);
	      pf0=pf;
	      pf->next=(WPatRange *)new(sizeof(WPatRange));
	      pf=pf->next;
	      pf->unit=punit;
	    }
	    break;
	  case 13: /* Port */
	  case 14: /* Ports */
	    if (nlist < 2 || nlist > 14) 
	      print_error(opt->confile,lineno, plist,
			  "Unknown port parameters specified");
	    else {
	      p=plist+strlen(plist)+1;
	      if (strcasecmp(p,"ALL") == 0) iport=ALL;
	      else {
		if (IsCharDigit(*p) == NO)
		  print_error(opt->confile,lineno,p,
			      "Unknown keyword specified");
		port_INDX=atoi(p);
		iport=IsValueInVec(port_INDX,td->port_indx,td->jd->nport);
		if (iport == NO) 
		  print_error(opt->confile,lineno,p,
			      "Invalid port number specified");
	      }
	      for (i=0; i < nlist-2; i++) {
		p=p+strlen(p)+1;
		if (strcasecmp(p,"MAG") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p);
		  SetPortPara(td->jd,NEW_VM,iport,value);
		  i++;
		  iport_mag=YES;
		}
		if (strcasecmp(p,"PHASE") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p)*DEG_2_RAD;
		  SetPortPara(td->jd,NEW_PH,iport,value);
		  i++;
		  iport_phase=YES;
		}
		if (strcasecmp(p,"R") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p);
		  SetPortPara(td->jd,NEW_R,iport,value);
		  i++;
		}
		if (strcasecmp(p,"X") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p);
		  SetPortPara(td->jd,NEW_X,iport,value);
		  i++;
		}
		if (strcasecmp(p,"L") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p)*1e-9;
		  SetPortPara(td->jd,NEW_L,iport,value);
		  i++;
		}
		if (strcasecmp(p,"C") == 0 ) {
		  p=p+strlen(p)+1;
		  if (IsCharDigit(*p) == NO)
		    print_error(opt->confile,lineno,p,
				"Unknown keyword specified");
		  value=atof(p)*1e-12;
		  SetPortPara(td->jd,NEW_C,iport,value);
		  i++;
		}
	      }
	      if (iport_mag == NO ) 
		print_error(opt->confile,lineno,"PORT",
			    "No port voltage MAGnitude is specified");
	      else   iport_mag=NO;
	      if (iport_phase == NO ) 
		print_error(opt->confile,lineno,"PORT",
			    "No port voltage PHASE is specified");
	      else   iport_phase=NO;
	    }
	    break;
	  } /* switch i */
	  break;
	} /* if ccompare */
      } /* for keyword */
      if (keyword[i] == NULL) 
	print_error(opt->confile,lineno,plist,"Unknown keyword specified");
    } /* if noncommented line */
  } /* while, each line */
  if (pf0 != NULL) {
    sFree((char *)pf0->next);
    pf0->next=NULL;
    pf0=NULL;
  }
  /* check if voltage is specified for each port */
  CheckPortVolt(opt,td->jd);

  /* check if freq is specifed */
  pf=td->freq;
  if (pf == NULL || !Bit2(pf->unit)) {
    fprintf(stderr,"File %s : %s\n",
	    opt->confile,"No frequency is specified");
    exit(1);
  }

  /* check if ang is specifed */
  if (td->ang1.unit == ANG_DEGREE) {
    fprintf(stderr,"File %s : %s\n",
	    opt->confile,"ANG1 is not specified");
    exit(1);
  }
  if (td->ang2.unit == ANG_DEGREE) {
    fprintf(stderr,"File %s : %s\n",
	    opt->confile,"ANG2 is not specified");
    exit(1);
  }
  
    
  sFree((char *)s1);
  sFree((char *)s2);
  sFree((char *)s3);
  sFree((char *)list);
} /* process_confile */

 print_error(file,line,plist,comments)
     char *file;
     int line;
     char *plist;
     char *comments;
{
  fprintf(stderr,"File %s (line %d,keyword %s): %s\n",
	  file,line,plist,comments);
  exit(1);
}

 sort_port_indx(td)
     WPatGenTmp *td;
{
  port_typ *pport;
  int i;

  td->port_indx=(int *)new(sizeof(int)*td->jd->nport);
  pport=td->jd->cir->ports->port0;
  for (i=0; i < td->jd->nport; i++) {
    while (IsValueInVec(iabs(pport->xgeom_index),td->port_indx,i) != NO)
      pport++;
    *(td->port_indx+i)=pport->xgeom_index;
    *(td->jd->rsrc+i)=pport->rp;
    *(td->jd->xsrc+i)=pport->xp;
    *(td->jd->lsrc+i)=pport->lp;
    *(td->jd->csrc+i)=pport->cp;
  }
}

int  iabs(i)
     int i;
{
  if (i < 0) return (-i);
  else       return (i);
}

 int IsValueInVec(ivalue,ivec,nvec)
     int ivalue;
     int *ivec;
     int nvec;
{
  int i;

  for (i=0; i < nvec; i++) {
    if (ivec[i] == ivalue ) return (i);
  }
  return (NO);
}

 int  IsDValueInVec(value,vec,nvec)
     double value;
     double *vec;
     int nvec;
{
  int i;

  for (i=0; i < nvec; i++) {
    if (fabs(vec[i]-value) <= 0.001) return (i);
  }
  return (NO);
}

 char *PatCoordNames[]={
  "SPHERE",
  "TOP",
  "EARTH",
  "ARRAY",
  "SINE",
   NULL
};
    
CheckCoordName(p,td)
     char *p;
     WPatGenTmp *td;
{
  int i;

  for (i=0; PatCoordNames[i] != NULL; i++) {
    if (strcasecmp(p,PatCoordNames[i]) == 0) {
      td->icoord=i;
      return;
    }
  }
  td->icoord=NO;
}

CheckAngleRange(file,lineno,td,iangle)
     char *file;
     int lineno;
     WPatGenTmp *td;
     int iangle;
{
  float tt,d;
  WPatRange *ang;

  if (iangle == 0) ang=&(td->ang1);
  else             ang=&(td->ang2);
  CheckRange(ang);
  d=ang->stop-ang->start;
  switch(td->icoord) {
  case PTN_COORD_SPHERICAL:
    if (iangle == 0) {
      if (d > 180.0 || ang->start < 0.0 || ang->stop > 180.0) {
	fprintf(stderr,"File %s (line %d): Invalid ANG1 Range.\n",file,lineno);
	fprintf(stderr,"  Valid ANG1 Range in Spherical Coordinates: 0 180\n");
	exit(1);
      }
    }
    else {
      if ( d > 360.0 || ang->start < -180.0 || ang->stop > 360.0) {
	fprintf(stderr,"File %s (line %d): Invalid ANG2 Range.\n",file,lineno);
	fprintf(stderr,"  Valid ANG2 Range in Spherical Coordinates: -180 180 or 0 360 \n");
	exit(1);
      }
    }
    break;
  case PTN_COORD_POLAR:
    if (d > 180.0 || ang->start < -90.0 || ang->stop > 90.0) {
      fprintf(stderr,"File %s (line %d): Invalid ANG%d Range.\n",file,lineno,iangle+1);
      fprintf(stderr,"  Valid ANG%d Range in Topview Coordinates: -90 90\n",iangle+1);
      exit(1);
    }
    break;
  case PTN_COORD_EARTH:
    if (d > 180.0 || ang->start < -90.0 || ang->stop > 90.0) {
      fprintf(stderr,"File %s (line %d): Invalid ANG%d Range.\n",file,lineno,iangle+1);
      fprintf(stderr,"  Valid ANG%d Range in Earthview Coordinates: -90 90\n",iangle+1);
      exit(1);
    }
    break;
  case PTN_COORD_ARRAY_SINE:
    if (d > 180.0 || ang->start < -90.0 || ang->stop > 90.0) {
      fprintf(stderr,"File %s (line %d): Invalid ANG%d Range.\n",file,lineno,iangle+1);
      fprintf(stderr,"  Valid ANG%d Range in Array Sine Coordinates: -90 90\n",iangle+1);
      exit(1);
    }
    break;
  case PTN_COORD_ARRAY:
    if (d > 2.0 || ang->start < -1.0 || ang->stop > 1.0) {
      fprintf(stderr,"File %s (line %d): Invalid ANG%d Range.\n",file,lineno,iangle+1);
      fprintf(stderr,"  Valid ANG%d Range in Array Coordinates: -1 1\n",iangle+1);
      exit(1);
    }
    break;
  }
}
    
 SetPortPara(jd,ipara,indx,value)
     Jdat *jd;
     int ipara;
     int indx;
     float value;
{
  int i;

  if (indx == ALL) {
    for (i=0; i < jd->nport; i++) 
      change_src(jd,ipara,i,(double)value);
  }
  else 
    change_src(jd,ipara,indx,(double)value);
}

WPatRange  *SetAllFreq(td,pf)
     WPatGenTmp *td;
     WPatRange *pf;
{
  WPatRange *p;
  int i;

  p=pf;
  for (i=0; i < td->jd->nfreq; i++) {
    p->unit=FREQ_MHZ;
    p->start= *(td->jd->freq+i);
    p->unit += 2;
    p->stop=p->start;
    p->unit += 4;
    p->step=0;
    p->unit += 8;
    if (i == td->jd->nfreq -1) 
      return (p);
    p->next=(WPatRange *)new(sizeof(WPatRange));
    p=p->next;
  }
}

WPatRange  *SetFirstFreq(td,pf)
     WPatGenTmp *td;
     WPatRange *pf;
{
  pf->start= *td->jd->freq;
  pf->stop=pf->start;
  pf->step=0.0;
  pf->unit=FREQ_MHZ;
  pf->unit += 14;
  pf->next=NULL;
  return(pf);
}

 CheckFreqRange(file,lineno,plist,pf,td)
     char *file;
     int lineno;
     char *plist;
     WPatRange *pf;
     WPatGenTmp *td;
{
  int i,nv;
  float *vf;

  CheckRange(pf);
  if (Bit1(pf->unit) == FREQ_GHZ)   {
    pf->start *= 1000.0;
    pf->stop  *= 1000.0;
    pf->step  *= 1000.0;
    pf->unit  -= 1;
  }
  vf=Range2FVector(pf,&nv);
  for (i=0; i < nv; i++) {
    if (IsDValueInVec((double)vf[i],td->jd->freq,td->jd->nfreq) == NO) 
      print_error(file,lineno,plist,"Invalid frequency value specified");
  }
  sFree((char *)vf);
}

 CheckVersion(version0,version)
     char *version0;
     char *version;
{
  char *p,*p0;
  
  p=version;
  p0=version0;
  while (*p != '\0' && *p0 != '\0') {
    if (*p < *p0) return(NO);
    p++; p0++;
  }

  return (YES);
}

CheckRange(pf)
     WPatRange *pf;
{
  float tt;

  /* if stop > start, swap stop & start */
  if (Bit3(pf->unit)) { /* stop is set */
    if (pf->stop < pf->start) {
      tt=pf->start;
      pf->start=pf->stop;
      pf->stop=tt;
    }
  }
  else 
    pf->stop=pf->start;

  /* if step < 0, swap start and stop */
  if (Bit4(pf->unit)) { 
    if (pf->step < 0.0 && pf->start > pf->stop ) {
      tt=pf->start;
      pf->start=pf->stop;
      pf->stop=tt;
    }
    pf->step = fabs(pf->step);
  }
  else 
    pf->step=pf->stop-pf->start;
  
  /* make step is valid */
  if (fabs((pf->stop - pf->start)) < EPS) 
    pf->step=0.0;

  /*This else was appended to above if, results in only two steps
    if stop is zero.  Don't know why that was thought to be an
    invalid stop.  Commented out jcr16aug93*/
  /*else if (fabs(pf->stop) < EPS) 
      pf->step=pf->stop-pf->start;*/

}

CheckPortVolt(opt,jd)
     WPatGenOpt *opt;
     Jdat *jd;
{
  int i;
  double tt;

  tt=0.0;
  for (i=0; i < jd->nport; i++) 
    tt += fabs(*(jd->vsrc_mg+i));
  
  if (tt < EPS) {
    fprintf(stderr,"Error: no voltage MAGnitude is specified.\n");
    exit(1);
  }
}  
