#include "local.h"
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "son.h"
#include "em.typ"
#include "jxy.typ"
#include "complex2.h"
#include "patdata.h"

static print_usage();
static print_usage2();

/*
 * parse(argc,argv)
 *
 *   parse command line 
 *
 */
void
  WPatGen_parse(argc,argv,opt)
int  argc;
char *argv[];
WPatGenOpt *opt;
{
  extern char *get_version();
  extern char *hid_sn();
  extern char *WStr_copy();
  extern char *WFile_ext();
  extern char *getpath_example();
  extern char *strrchr();
  char *p,*pargv;
  int i;

  opt->argc=argc;
  opt->argv=(char **)new(argc*sizeof(char *));
  for (i=0; i < argc; i++) 
      *(opt->argv+i)=WStr_copy(*(argv+i));

  opt->verbose=FALSE;
  opt->state_version=FALSE;
  opt->state_mem=FALSE;
  opt->state_freq=FALSE;
  opt->state_port=FALSE;
  opt->quick_2d=FALSE;
  opt->quick_3d=FALSE;
  opt->debug=FALSE;
  opt->apprx=0;
  opt->nray=0;
  opt->print_j=0;
  opt->precision=0; /* 1=single precision */
  opt->confile=NULL;
  opt->jxyfile=NULL;
  opt->patfile=NULL;

  if (argc < 2)    print_usage(argv[0],NULL);

  if ( check_version_option(argc,argv) ) {
    print_version(stdout);
    exit(0);
  }
      
  for (i=1; i < argc; i++) {
    pargv=*(argv+i);
    if (*pargv == '-') {
      p=pargv+1;
      while (*p != '\0') {
	switch (*p) {
	case 'h':
	  print_usage2(argv[0]);
	  break;
	case 'v':
	  opt->verbose=YES;
	  p++;
	  break;
	case 'N':
	  opt->state_mem=YES;
	  p++;
	  break;
	case 'f':
	  opt->state_freq=YES;
	  p++;
	  break;
	case 't':
	  opt->state_port=YES;
	  p++;
	  break;
	case 'd':
	  p++;
	  if (*p == '2') 	  opt->quick_2d=YES;
	  else if (*p == '3')     opt->quick_3d=YES;
	  else print_usage(argv[0],"Unknown option specified");
	  p++;
	  break;
	case 'D':
	  if (strcmp(p,"Debug") == 0) 
	    opt->debug=YES;
	  else print_usage(argv[0],"Unknown option specified");
	  p += 5;
	  break;
	case 'X':
	  if (opt->debug == FALSE) 
	    print_usage(argv[0],"Unknown option specified");
	  p++;
	  if (!isdigit(*p)) 
	    print_usage(argv[0],"Unknown option specified");
	  opt->apprx=atoi(p);
	  if (opt->apprx > 9) p += 2;
	  else                p++;
	  break;
	case 'R':
	  if (opt->debug == FALSE) 
	    print_usage(argv[0],"Unknown option specified");
	  p++;
	  if (!isdigit(*p)) 
	    print_usage(argv[0],"Unknown option specified");
	  opt->nray=atoi(p);
	  if (opt->nray > 9) p += 2;
	  else                p++;
	  break;
	case 'P':
	  if (opt->debug == FALSE) 
	    print_usage(argv[0],"Unknown option specified");
	  p++;
	  if (strcmp(p,"Jx") == 0) opt->print_j +=  1 ;
	  else if (strcmp(p,"Jy") == 0) opt->print_j += (1 << 1);
	  else if (strcmp(p,"Ez") == 0) opt->print_j  += ( 1 << 2);
	  else 	    print_usage(argv[0],"Unknown option specified");
	  p += 2;
	  break;
	case 'j':
	  p++;
	  i++;
	  if (i >= argc)
	    print_usage(argv[0],"Current file not specified");
	  opt->jxyfile=WStr_copy(argv[i]);
	  break;
	case 'p':
	  p++;
	  i++;
	  if (i >= argc) 
	    print_usage(argv[0],"Pattern output file not specified");
	  opt->patfile=WStr_copy(argv[i]);
	  break;
	case 'c':
	  p++;
	  i++;
	  if (i >= argc) 
	    print_usage(argv[0],"Antenna configuration file not specified");
	  opt->confile=WStr_copy(argv[i]);
	  break;
	default:
	  print_usage(argv[0],"Unknow options specified");
	  break;
	} /*switch *p */
      } /* while p */
    } /* if pargv[0]=='-' */
    else {
      p=strrchr(pargv,'.');
      if (p == NULL) {
	if (i == argc -1 && opt->jxyfile == NULL) 
	  opt->jxyfile=WFile_ext(pargv,"jxy");
	else
	  print_usage(argv[0],"Unkown options specified");
      }
      else {
	p++;
	if (strcmp(p,"jxy") == 0) 
	  opt->jxyfile=WStr_copy(pargv);
	else if (strcmp(p,"pat") == 0) 
	  opt->patfile=WStr_copy(pargv);
	else if (strcmp(p,"pg") == 0) 
	  opt->confile=WStr_copy(pargv);
	else if (i == argc -1) {
          opt->jxyfile=(char *)new(strlen(pargv)+4);
	  sprintf(opt->jxyfile,"%s.jxy",pargv);
	}
	else print_usage(argv[0],"Unknown options specified");
      }
    }
  }
  
  if (opt->quick_2d && opt->quick_3d) 
    print_usage(argv[0],"May not specified both d2 and d3 at the same time");

  if (opt->quick_2d)   {
    opt->confile=getpath_example(argv[0],"ctld2.pg");
  }
  if (opt->quick_3d)   {
    opt->confile=getpath_example(argv[0],"ctld3.pg");
  }

  if (opt->confile == NULL) opt->confile=WStr_copy("ctl.pg");
  if (opt->jxyfile == NULL) print_usage(argv[0],"Current file not specified");
  if (opt->patfile == NULL) {
    p=strrchr(opt->jxyfile,'.');
    if (p != NULL && strcmp(p,".jxy") == 0) 
      opt->patfile=WFile_ext(opt->jxyfile,"pat");
    else 
/* NOT FIXED IN 2.4a - SPARC - krs */
/* ORIGINAL LINE: print_usage(argv[0],"Pattern output file not specified"); */
      print_usage(argv[0],"Pattern output file not specified");
  }
  if (opt->verbose) {
    printf("Current distribution file -- %s --\n",opt->jxyfile);
    printf("Pattern generation control file -- %s --\n",opt->confile);
    printf("Pattern output file -- %s --\n", opt->patfile);
    printf("\n\n\n\n%40s\n\n","PATGEN");
    printf("%38s%5s\n\n\n","Version ",get_version());
    printf("%60s\n","(c) Copyright - 1993 Sonnet Software, Inc.");
    printf("%47s\n\n","All rights reserved");
  }
}

/*
 * void print_titlepage(WPatGenOpt *opt,WPatGenTmp *td)
 *  print out information in verbose mode
 *
 */
void
  print_titlepage(opt,td)
WPatGenOpt *opt;
WPatGenTmp *td;
{
  printf("Computing %d data points, %d Mbytes.\n",td->npt,td->nbyte);
}

static print_usage(progname,comments)
char *progname;
char *comments;
{
  if (comments) printf("%10s: %s\n\n",progname,comments);
  printf("%s [-Ver] [-h] [-d2] [-d3] [-f] [-t] [-v] [-N] [-j] file.jxy [-g] [ctl.pg] [-p] [file.pat]\n\n",
	 progname);
  exit(1);
}
  
static print_usage2(progname)
char *progname;
{
  printf("%10s: [options] file.jxy\n",progname);
  printf("   options: \n");
  printf("      -Ver  get current version\n");
  printf("        -h  this help message\n");
  printf("        -d2 quick 2d calculation\n");
  printf("        -d3 quick 3d calculation\n");
  printf("        -f  list all frequencies\n");
  printf("        -t  list port characteristics\n");
  printf("        -v  verbose\n");
  printf("        -N  estimate memory requirement\n");
  printf("   -j file  explicitly give jxy file\n");
  printf("   -g file  explicitly give pg file\n");
  printf("   -p file  explicitly give pat file\n");
  exit (0);
}
    
/*
 * char *getpath_example(char *file)
 *  Get full path of file to $SONNET_DIR/examples/file
 *    If successful, return the file with a full path name
 *    if not,        print out error message then terminate
 */
char 
  *getpath_example(program_name,filename)
char *program_name;
char *filename;
{
  char *path;
  FILE *file;
  char *sonnet_dir;
  char *msg;

  sonnet_dir = get_sonnet_dir(FALSE);
  if ( sonnet_dir == NULL ) {
    print_usage(program_name,"SONNET_DIR variable not defined");
  }
  else {
    path = new( sizeof(char) * MAXPATHLEN);
    strncpy(path, sonnet_dir, MAXPATHLEN);
    filename_cat(path, "examples", (char *) NULL);
    filename_cat(path, filename, (char *) NULL);
  }
  file = fopen(path,"r");
  if ( file == NULL ) {
    msg = new( sizeof(char) * strlen(path)+30);
    sprintf(msg,"Can not read %s",path);
  }
  fclose(file);
  return(path);
}
